var Presentation = OZ.Class();
Presentation.init = function() {
	Presentation.presentation = new Presentation();
}

Presentation.prototype.init = function() {
	this.slides = [];
	this.index = -1;
	this.select = OZ.$("slide-select");
	this.counter = OZ.$("slide-counter");
	this.sizes = ["50%","75%","90%","100%","120%","150%","200%","250%","350%"];
	this.size = -1;
	this.title = document.title;
	
	this.anchor = this.bind(this.anchor);
	this.build();
	this.normal();
	
	this.selectChange = this.bind(this.selectChange);
	this.key = this.bind(this.key);
	this.next = this.bind(this.next);
	this.click = this.bind(this.click);
	if (this.select) {
		OZ.Event.add(this.select, "change", this.selectChange);
		OZ.Event.add(this.select, "click", OZ.Event.stop);
	}
	OZ.Event.add(document, "keydown", this.key);
	if (this.counter) { OZ.Event.add(this.counter, "click", this.click); }
	
	var index = 0;
	var re = location.hash.match(/#?([0-9]+)$/)
	if (re) { index = parseInt(re[1])-1; }
	this.goTo(index);
}

Presentation.prototype.build = function() {
	var all = document.getElementsByTagName("*");
	for (var i=0;i<all.length;i++) {
		var elm = all[i];
		if (OZ.DOM.hasClass(elm, "slide")) {
			var s = new Slide(this, elm);
			this.slides.push(s);
		}
	}
	
	if (this.select) { this.buildSelect(); }
	this.slides.forEach(function($){ $.hide(); });
	
	var links = document.getElementsByTagName("a");
	for (var i=0;i<links.length;i++) {
		OZ.Event.add(links[i], "click", this.anchor);
	}
}

Presentation.prototype.buildSelect = function() {
	var groups = [];
	var ch = OZ.$("slide-chapters");
	if (ch) {
		var lis = ch.getElementsByTagName("li");
		for (var i=0;i<lis.length;i++) { groups.push(lis[i].innerHTML); }
	}
	
	var parent = this.select;
	var currentChapter = false;
	
	for (var i=0;i<this.slides.length;i++) {
		var slide = this.slides[i];
		var ch = slide.getChapter();
		
		if (ch) {
			if (ch != currentChapter) {
				currentChapter = ch;
				parent = OZ.DOM.elm("optgroup");
				this.select.appendChild(parent);
				parent.label = groups[ch-1];
			}
		} else {
			parent = this.select;
		}
		
		var o = OZ.DOM.elm("option");
		var t = (i+1) + ". " + slide.getTitle();
		o.innerHTML = t;
		o.value = i;
		parent.appendChild(o);
	}
}

Presentation.prototype.goTo = function(index, expand) {
	if (index < 0 || index >= this.slides.length) { return; }
	if (this.index != -1) {
		this.slides[this.index].hide();
	}
	this.index = index;
	
	if (this.select) { 
		this.select.selectedIndex = this.index; 
		/* ie hack... */
		this.select.getElementsByTagName("option")[index].setAttribute("selected","selected");
	}
	if (this.counter) { this.counter.innerHTML = (this.index+1) + "/" + this.slides.length; }
	document.title = this.title + " #"+(this.index+1);
	location.hash = "#"+(this.index+1);
	this.slides[this.index].show(expand);
}

Presentation.prototype.selectChange = function(e) {
	this.goTo(this.select.selectedIndex, false);
}

Presentation.prototype.click = function(e) {
	if (e.which && e.which != 1) { return; }
	if (OZ.ie && e.button) { return; }
	this.next();
}

Presentation.prototype.key = function(e) {	
	switch (e.keyCode) {
		case 37:
		case 33: this.prev(); break;
		
		case 32:
		case 39:
		case 34: this.next(); break;
		
		case 36: this.goTo(0); break;
		case 35: this.goTo(this.slides.length-1); break;
		
		case 66: this.bigger(); break;
		case 78: this.normal(); break;
		case 83: this.smaller(); break;
		case 191: this.help();	break;
	}
}

Presentation.prototype.next = function() {
	var result = this.slides[this.index].next();
	if (!result) {
		this.goTo(this.index+1, false);
	}
}

Presentation.prototype.prev = function() {
	var result = this.slides[this.index].prev();
	if (!result) {
		this.goTo(this.index-1, true);
	}
}

Presentation.prototype.help = function() {
	var arr = [];
	arr.push("Font size - B/S/N");
	arr.push("Previous - Left/PgUp");
	arr.push("Next - Right/PgDown/Space");
	arr.push("First - Home");
	arr.push("Last - End");
	arr.push("");
	arr.push("Slides: http://ondras.zarovi.cz/slides/");
	arr.push("© 2009 Ondřej Žára");
	alert(arr.join("\n"));
}

Presentation.prototype.smaller = function() {
	if (this.size > 0) {
		this.size--;
		document.body.style.fontSize = this.sizes[this.size];
	}
}

Presentation.prototype.bigger = function() {
	if (this.size+1 < this.sizes.length) {
		this.size++;
		document.body.style.fontSize = this.sizes[this.size];
	}
}

Presentation.prototype.normal = function() {
	this.size = this.sizes.indexOf("100%");
	document.body.style.fontSize = this.sizes[this.size];
}

Presentation.prototype.anchor = function(e) {
	var elm = OZ.Event.target(e);
	if (!elm.href) { return; }
	var re = elm.href.toString().match(/#([0-9])+/);
	if (re) {
		var num = parseInt(re[1]) - 1;
		this.goTo(num);
	}
}

var Slide = OZ.Class();

Slide.prototype.init = function(owner, elm) {
	this.owner = owner;
	this.elm = elm;
	this.hiddens = [];
	this.ptr = -1;
	
	this.findhiddens(this.elm);
}


Slide.prototype.findhiddens = function(node) {
	var children = node.childNodes;
	var is = OZ.DOM.hasClass(node,"slide-hide");
	
	var hasNodes = false;
	for (var i=0;i<children.length;i++) {
		var child = children[i];
		if (child.nodeType == 1) { hasNodes = true; }
	}
	if (is && !hasNodes) { this.hiddens.push(node); }
	
	for (var i=0;i<children.length;i++) {
		var child = children[i];
		if (child.nodeType != 1) { continue; }
		if (is) { this.hiddens.push(child); }
		this.findhiddens(child);
	}
}

Slide.prototype.hide = function() {
	OZ.DOM.addClass(this.elm, "slide-none");
}

Slide.prototype.show = function(expand) {
	OZ.DOM.removeClass(this.elm, "slide-none");
	for (var i=0;i<this.hiddens.length;i++) {
		var node = this.hiddens[i];
		OZ.DOM.removeClass(node, "slide-hidden");
		if (!expand) {
			OZ.DOM.addClass(node, "slide-hidden");
		}
	}
	this.ptr = (expand ? this.hiddens.length-1 : -1);
}

Slide.prototype.next = function() {
	if (this.ptr+1 < this.hiddens.length) {
		this.ptr++;
		OZ.DOM.removeClass(this.hiddens[this.ptr],"slide-hidden");
		return true;
	} else { return false; }
}

Slide.prototype.prev = function() {
	if (this.ptr > -1) {
		OZ.DOM.addClass(this.hiddens[this.ptr],"slide-hidden");
		this.ptr--;
		return true;
	} else { return false; }
}

Slide.prototype.getTitle = function() {
	var list = ["h1","h2","h3"];
	for (var i=0;i<list.length;i++) {
		var all = this.elm.getElementsByTagName(list[i]);
		if (all.length) { return all[0].innerText || all[0].textContent; }
	}
	return "";
}

Slide.prototype.getChapter = function() {
	var cn = this.elm.className;
	var r = cn.match(/slide-chapter-([0-9]+)/);
	if (!r) return false;
	return r[1];
}

OZ.Event.add(window, "load", Presentation.init);
